@@ -15,11 +15,14 @@ Files that generate images should start with 'plot'.
 from __future__ import division, print_function, absolute_import
 from time import time
 import copy
+import contextlib
 import ast
 import codecs
 import gc
 import os
 import re
+import warnings
+from shutil import copyfile
 import subprocess
 import sys
 import traceback
@@ -27,7 +30,8 @@ import codeop
 from io import StringIO
 from distutils.version import LooseVersion
 
-from .scrapers import save_figures, ImagePathIterator, clean_modules
+from .scrapers import (save_figures, ImagePathIterator, clean_modules,
+                       _find_image_ext)
 from .utils import replace_py_ipynb, scale_image, get_md5sum, _replace_md5
 
 # Try Python 2 first, otherwise load from Python 3
@@ -36,7 +40,7 @@ try:
     from textwrap import indent
 except ImportError:
     def indent(text, prefix, predicate=None):
-        """Adds 'prefix' to the beginning of selected lines in 'text'.
+        """Add 'prefix' to the beginning of selected lines in 'text'.
 
         If 'predicate' is provided, 'prefix' will only be added to the lines
         where 'predicate(line)' is True. If 'predicate' is not provided,
@@ -52,6 +56,9 @@ except ImportError:
                 yield (prefix + line if predicate(line) else line)
         return ''.join(prefixed_lines())
 
+    FileNotFoundError = IOError
+
+
 import sphinx
 
 from . import glr_path_static
@@ -59,7 +66,7 @@ from . import sphinx_compatibility
 from .backreferences import write_backreferences, _thumbnail_div
 from .downloads import CODE_DOWNLOAD
 from .py_source_parser import (split_code_and_text_blocks,
-                               get_docstring_and_rest)
+                               get_docstring_and_rest, remove_config_comments)
 
 from .notebook import jupyter_notebook, save_notebook
 from .binder import check_binder_conf, gen_binder_rst
@@ -170,7 +177,7 @@ SPHX_GLR_SIG = """\n
 
  .. rst-class:: sphx-glr-signature
 
-    `Gallery generated by Sphinx-Gallery <https://sphinx-gallery.readthedocs.io>`_\n"""  # noqa: E501
+    `Gallery generated by Sphinx-Gallery <https://sphinx-gallery.github.io>`_\n"""  # noqa: E501
 
 
 def codestr2rst(codestr, lang='python', lineno=None):
@@ -189,9 +196,31 @@ def codestr2rst(codestr, lang='python', lineno=None):
     return code_directive + indented_block
 
 
-def extract_intro_and_title(filename, docstring):
-    """ Extract the first paragraph of module-level docstring. max:95 char"""
+def _regroup(x):
+    x = x.groups()
+    return x[0] + x[1].split('.')[-1] + x[2]
 
+
+def _sanitize_rst(string):
+    """Use regex to remove at least some sphinx directives."""
+    # :class:`a.b.c <thing here>`, :ref:`abc <thing here>` --> thing here
+    p, e = r'(\s|^):[^:\s]+:`', r'`(\W|$)'
+    string = re.sub(p + r'\S+\s*<([^>`]+)>' + e, r'\1\2\3', string)
+    # :class:`~a.b.c` --> c
+    string = re.sub(p + r'~([^`]+)' + e, _regroup, string)
+    # :class:`a.b.c` --> a.b.c
+    string = re.sub(p + r'([^`]+)' + e, r'\1\2\3', string)
+
+    # ``whatever thing`` --> whatever thing
+    p = r'(\s|^)`'
+    string = re.sub(p + r'`([^`]+)`' + e, r'\1\2\3', string)
+    # `whatever thing` --> whatever thing
+    string = re.sub(p + r'([^`]+)' + e, r'\1\2\3', string)
+    return string
+
+
+def extract_intro_and_title(filename, docstring):
+    """Extract and clean the first paragraph of module-level docstring."""
     # lstrip is just in case docstring has a '\n\n' at the beginning
     paragraphs = docstring.lstrip().split('\n\n')
     # remove comments and other syntax like `.. _link:`
@@ -216,9 +245,9 @@ def extract_intro_and_title(filename, docstring):
     intro_paragraph = title if len(paragraphs) < 2 else paragraphs[1]
     # Concatenate all lines of the first paragraph and truncate at 95 chars
     intro = re.sub('\n', ' ', intro_paragraph)
+    intro = _sanitize_rst(intro)
     if len(intro) > 95:
         intro = intro[:95] + '...'
-
     return intro, title
 
 
@@ -255,7 +284,8 @@ def save_thumbnail(image_path_template, src_file, file_conf, gallery_conf):
         raise TypeError(
             'sphinx_gallery_thumbnail_number setting is not a number, '
             'got %r' % (thumbnail_number,))
-    thumbnail_image_path = image_path_template.format(thumbnail_number)
+    thumbnail_image_path, ext = _find_image_ext(image_path_template,
+                                                thumbnail_number)
 
     thumb_dir = os.path.join(os.path.dirname(thumbnail_image_path), 'thumb')
     if not os.path.exists(thumb_dir):
@@ -263,7 +293,7 @@ def save_thumbnail(image_path_template, src_file, file_conf, gallery_conf):
 
     base_image_name = os.path.splitext(os.path.basename(src_file))[0]
     thumb_file = os.path.join(thumb_dir,
-                              'sphx_glr_%s_thumb.png' % base_image_name)
+                              'sphx_glr_%s_thumb.%s' % (base_image_name, ext))
 
     if src_file in gallery_conf['failing_examples']:
         img = os.path.join(glr_path_static(), 'broken_example.png')
@@ -275,18 +305,34 @@ def save_thumbnail(image_path_template, src_file, file_conf, gallery_conf):
         img = gallery_conf.get("default_thumb_file", img)
     else:
         return
-    scale_image(img, thumb_file, *gallery_conf["thumbnail_size"])
+    if ext == 'svg':
+        copyfile(img, thumb_file)
+    else:
+        scale_image(img, thumb_file, *gallery_conf["thumbnail_size"])
+
+
+def _get_readme(dir_, gallery_conf, raise_error=True):
+    extensions = ['.txt'] + sorted(gallery_conf['app'].config['source_suffix'])
+    for ext in extensions:
+        fname = os.path.join(dir_, 'README' + ext)
+        if os.path.isfile(fname):
+            return fname
+    if raise_error:
+        raise FileNotFoundError(
+            "Example directory {0} does not have a README file with one "
+            "of the expected file extensions {1}. Please write one to "
+            "introduce your gallery.".format(dir_, extensions))
+    return None
 
 
 def generate_dir_rst(src_dir, target_dir, gallery_conf, seen_backrefs):
-    """Generate the gallery reStructuredText for an example directory"""
-
+    """Generate the gallery reStructuredText for an example directory."""
     head_ref = os.path.relpath(target_dir, gallery_conf['src_dir'])
     fhindex = """\n\n.. _sphx_glr_{0}:\n\n""".format(
         head_ref.replace(os.path.sep, '_'))
 
-    with codecs.open(os.path.join(src_dir, 'README.txt'), 'r',
-                     encoding='utf-8') as fid:
+    fname = _get_readme(src_dir, gallery_conf)
+    with codecs.open(fname, 'r', encoding='utf-8') as fid:
         fhindex += fid.read()
     # Add empty lines to avoid bug in issue #165
     fhindex += "\n\n"
@@ -318,7 +364,8 @@ def generate_dir_rst(src_dir, target_dir, gallery_conf, seen_backrefs):
         clean_modules(gallery_conf, fname)
         src_file = os.path.normpath(os.path.join(src_dir, fname))
         computation_times.append((time_elapsed, src_file))
-        this_entry = _thumbnail_div(build_target_dir, fname, intro) + """
+        this_entry = _thumbnail_div(target_dir, gallery_conf['src_dir'],
+                                    fname, intro) + """
 
 .. toctree::
    :hidden:
@@ -341,6 +388,7 @@ def generate_dir_rst(src_dir, target_dir, gallery_conf, seen_backrefs):
 
 
 def handle_exception(exc_info, src_file, script_vars, gallery_conf):
+    from .gen_gallery import _expected_failing_examples
     etype, exc, tb = exc_info
     stack = traceback.extract_tb(tb)
     # Remove our code from traceback:
@@ -353,8 +401,13 @@ def handle_exception(exc_info, src_file, script_vars, gallery_conf):
         traceback.format_list(stack) +
         traceback.format_exception_only(etype, exc))
 
-    logger.warning('%s failed to execute correctly: %s', src_file,
-                   formatted_exception)
+    expected = src_file in _expected_failing_examples(gallery_conf)
+    if expected:
+        func, color = logger.info, 'blue',
+    else:
+        func, color = logger.warning, 'red'
+    func('%s failed to execute correctly: %s', src_file,
+         formatted_exception, color=color)
 
     except_rst = codestr2rst(formatted_exception, lang='pytb')
 
@@ -368,6 +421,35 @@ def handle_exception(exc_info, src_file, script_vars, gallery_conf):
     return except_rst
 
 
+# Adapted from github.com/python/cpython/blob/3.7/Lib/warnings.py
+def _showwarning(message, category, filename, lineno, file=None, line=None):
+    if file is None:
+        file = sys.stderr
+        if file is None:
+            # sys.stderr is None when run with pythonw.exe:
+            # warnings get lost
+            return
+    text = warnings.formatwarning(message, category, filename, lineno, line)
+    try:
+        file.write(text)
+    except OSError:
+        # the file (probably stderr) is invalid - this warning gets lost.
+        pass
+
+
+@contextlib.contextmanager
+def patch_warnings():
+    """Patch warnings.showwarning to actually write out the warning."""
+    # Sphinx or logging or someone is patching warnings, but we want to
+    # capture them, so let's patch over their patch...
+    orig_showwarning = warnings.showwarning
+    try:
+        warnings.showwarning = _showwarning
+        yield
+    finally:
+        warnings.showwarning = orig_showwarning
+
+
 class _exec_once(object):
     """Deal with memory_usage calling functions more than once (argh)."""
 
@@ -379,7 +461,8 @@ class _exec_once(object):
     def __call__(self):
         if not self.run:
             self.run = True
-            exec(self.code, self.globals)
+            with patch_warnings():
+                exec(self.code, self.globals)
 
 
 def _memory_usage(func, gallery_conf):
@@ -389,7 +472,10 @@ def _memory_usage(func, gallery_conf):
         assert callable(func)
         mem, out = memory_usage(func, max_usage=True, retval=True,
                                 multiprocess=True)
-        mem = mem[0]
+        try:
+            mem = mem[0]  # old MP always returned a list
+        except TypeError:  # 'float' object is not subscriptable
+            pass
     else:
         out = func()
         mem = 0
@@ -406,8 +492,14 @@ def _get_memory_base(gallery_conf):
         sleep, timeout = (1, 2) if sys.platform == 'win32' else (0.5, 1)
         proc = subprocess.Popen(
             [sys.executable, '-c',
-             'import time, sys; time.sleep(%s), sys.exit(0)' % sleep])
-        memory_base = max(memory_usage(proc, interval=1e-3, timeout=timeout))
+             'import time, sys; time.sleep(%s); sys.exit(0)' % sleep],
+            close_fds=True)
+        memories = memory_usage(proc, interval=1e-3, timeout=timeout)
+        kwargs = dict(timeout=timeout) if sys.version_info >= (3, 5) else {}
+        proc.communicate(**kwargs)
+        # On OSX sometimes the last entry can be None
+        memories = [mem for mem in memories if mem is not None] + [0.]
+        memory_base = max(memories)
     return memory_base
 
 
@@ -422,18 +514,18 @@ def execute_code_block(compiler, block, example_globals,
 
     cwd = os.getcwd()
     # Redirect output to stdout and
-    orig_stdout = sys.stdout
+    orig_stdout, orig_stderr = sys.stdout, sys.stderr
     src_file = script_vars['src_file']
 
     # First cd in the original example dir, so that any file
     # created by the example get created in this directory
 
-    my_stdout = MixedEncodingStringIO()
+    captured_std = MixedEncodingStringIO()
     os.chdir(os.path.dirname(src_file))
 
     sys_path = copy.deepcopy(sys.path)
     sys.path.append(os.getcwd())
-    sys.stdout = LoggingTee(my_stdout, logger, src_file)
+    sys.stdout = sys.stderr = LoggingTee(captured_std, logger, src_file)
 
     try:
         dont_inherit = 1
@@ -447,7 +539,8 @@ def execute_code_block(compiler, block, example_globals,
             gallery_conf)
     except Exception:
         sys.stdout.flush()
-        sys.stdout = orig_stdout
+        sys.stderr.flush()
+        sys.stdout, sys.stderr = orig_stdout, orig_stderr
         except_rst = handle_exception(sys.exc_info(), src_file, script_vars,
                                       gallery_conf)
         # python2.7: Code was read in bytes needs decoding to utf-8
@@ -464,22 +557,23 @@ def execute_code_block(compiler, block, example_globals,
         mem = 0
     else:
         sys.stdout.flush()
-        sys.stdout = orig_stdout
+        sys.stderr.flush()
+        sys.stdout, orig_stderr = orig_stdout, orig_stderr
         sys.path = sys_path
         os.chdir(cwd)
 
-        my_stdout = my_stdout.getvalue().strip().expandtabs()
-        if my_stdout:
-            stdout = CODE_OUTPUT.format(indent(my_stdout, u' ' * 4))
+        captured_std = captured_std.getvalue().expandtabs()
+        if captured_std:
+            captured_std = CODE_OUTPUT.format(indent(captured_std, u' ' * 4))
         else:
-            stdout = ''
+            captured_std = ''
         images_rst = save_figures(block, script_vars, gallery_conf)
-        code_output = u"\n{0}\n\n{1}\n\n".format(images_rst, stdout)
+        code_output = u"\n{0}\n\n{1}\n\n".format(images_rst, captured_std)
 
     finally:
         os.chdir(cwd)
         sys.path = sys_path
-        sys.stdout = orig_stdout
+        sys.stdout, sys.stderr = orig_stdout, orig_stderr
     script_vars['memory_delta'].append(mem)
 
     return code_output
@@ -544,6 +638,7 @@ def execute_script(script_blocks, script_vars, gallery_conf):
         '__name__': '__main__',
         # Don't ever support __file__: Issues #166 #212
     }
+    script_vars['example_globals'] = example_globals
 
     argv_orig = sys.argv[:]
     if script_vars['execute_script']:
@@ -629,6 +724,13 @@ def generate_file_rst(fname, target_dir, src_dir, gallery_conf):
         'target_file': target_file}
 
     file_conf, script_blocks = split_code_and_text_blocks(src_file)
+
+    if gallery_conf['remove_config_comments']:
+        script_blocks = [
+            (label, remove_config_comments(content), line_number)
+            for label, content, line_number in script_blocks
+        ]
+
     output_blocks, time_elapsed = execute_script(script_blocks,
                                                  script_vars,
                                                  gallery_conf)
@@ -701,7 +803,8 @@ def rst_blocks(script_blocks, output_blocks, file_conf, gallery_conf):
                     example_rst += "\n\n|\n\n"
                 example_rst += code_rst
         else:
-            example_rst += bcontent + '\n'
+            block_separator = '\n\n' if not bcontent.endswith('\n') else '\n'
+            example_rst += bcontent + block_separator
     return example_rst
 
 
